class Admin::ActivitiesController < OrbitMemberController
	include Admin::ActivitiesHelper
	layout "member_plugin"
	before_action :set_activity, only: [:show, :edit , :update, :destroy]
	before_action :set_plugin
	before_action :get_settings,:only => [:new, :edit, :setting]

	before_action :need_access_right
	before_action :allow_admin_only, :only => [:index, :setting]

	def index
		if params[:sort].present?
			@activities = Activity.order_by(sort).page(params[:page]).per(10)
		else
			@activities = Activity.sort_hash.page(params[:page]).per(10)
		end
	end
	def sort
		case params[:sort]
		when "status"
			@sort = [[:is_top, params[:order]],
					[:is_hot, params[:order]],
					[:is_hidden,params[:order]],
					[:id,params[:order]]]
		when "category"
			@sort = {:category_id=>params[:order]}.merge({:id=>params[:order]})
		else
			if params[:sort].present?
				s = params[:sort].to_s
				@sort = {s=>params[:order]}.merge({:id=>params[:order]})
			else
				@sort = {}
			end
		end
		@sort
	end
	def new
		@member = MemberProfile.find_by(:uid=>params[:uid].to_s) rescue nil
		@activity = Activity.new
	end

	def create
		activity = Activity.create(activity_params)
		redirect_to params[:referer_url]
	end
	
	def show
	end
	def analysis
	end
	def analysis_report
		role = params[:role_id]
		year_start = params[:year_start]
		year_end = params[:year_end]
		graph_by = params[:graph_by]

		@data = get_chart_data(year_start,year_end,role,params[:graph_by],params[:time_zone])

		render :layout => false
	end
	
	def download_excel
		year_start = params[:year_start]
		year_end = params[:year_end]
		@data = get_data_for_excel(year_start,year_end,params[:time_zone])
		@protocol = (request.referer.blank? ? "http" : URI(request.referer).scheme)
		@host_url = "#{@protocol}://#{request.host_with_port}"
		respond_to do |format|
			format.xlsx {
				response.headers['Content-Disposition'] = 'attachment; filename="activities.xlsx"'
			}
		end
	end

	def edit
	end

	def destroy
		@activity.destroy
		redirect_to admin_activities_path(:page => params[:page])
	end

	def update
		@activity.update_attributes(activity_params)
		@activity.save
		redirect_to params[:referer_url]
	end


	def setting
	end

	def frontend_setting
		@member = MemberProfile.find_by(:uid=>params[:uid].to_s) rescue nil
		@intro = ActivityIntro.find_by(:member_profile_id=>@member.id) rescue nil
		@intro = @intro.nil? ? ActivityIntro.new({:member_profile_id=>@member.id}) : @intro
	end

	def update_frontend_setting
		@member = MemberProfile.find(intro_params['member_profile_id']) rescue nil
		@intro = ActivityIntro.find_by(:member_profile_id=>@member.id) rescue nil
		@intro = @intro.nil? ? ActivityIntro.new({:member_profile_id=>@member.id}) : @intro
		@intro.update_attributes(intro_params)
		@intro.save
		redirect_to URI.encode('/admin/members/'+@member.to_param+'/Activity')
	end

	def toggle_hide
		if params[:ids]
			@projects = Activity.any_in(_id: params[:ids])

			@projects.each do |project|
			project.is_hidden = params[:disable]
			project.save
			end
		end

		render json: {"success"=>true}
	end

	private 

	def activity_params
		params.require(:activity).permit!
	end

	def intro_params
		params.require(:activity_intro).permit! rescue nil
	end
	
	def get_settings
	end

	def set_plugin
		@plugin = OrbitApp::Plugin::Registration.all.select{|plugin| plugin.app_name.eql? 'Activity'}.first
	end

	def set_activity
		path = request.path.split('/')
		if path.last.include? '-'
			uid = path[-1].split("-").last
			uid = uid.split("?").first
		else
			uid = path[-2].split("-").last
			uid = uid.split("?").first
		end
		@activity = Activity.find_by(:uid => uid) rescue Activity.find(params[:id])
	end
end